-- ************************************************************************** --
-- * TitanSpec.lua                                                          * --
-- *                                                                        * --
-- * By: Rothpada                                                           * --
-- ************************************************************************** --

local TITAN_SPEC_ID = "TitanSpec";
local AceTimer = LibStub("AceTimer-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale("TitanSpec", true)

local TitanSpec_IS_DEBUGGING = false;

-- ************************************************************************** --
-- ******************************** Titan Functions ************************* --
-- ************************************************************************** --

function TitanPanelTitanSpecButton_OnLoad(self)
	self.registry = {
		id = TITAN_SPEC_ID,
		menuText = L["TITAN_SPEC_MENU_TEXT"],
		buttonTextFunction = "TitanPanelTitanSpecButton_GetButtonText",
		tooltipTitle = L["TITAN_SPEC_TOOLTIP_TITLE"],
		tooltipTextFunction = "TitanPanelTitanSpecButton_GetTooltipText",
		frequency = 1.0,
		savedVariables = {
			ShowLabelText = 1,
			ShowNumbers = 1,
			SpecText = {},
			SpecTree = {},
			BuildText = {},
		}	
	};

	AceTimer.ScheduleRepeatingTimer("TitanPanel"..TITAN_SPEC_ID, TitanPanelPluginHandle_OnUpdate, 1, {TITAN_SPEC_ID, TITAN_PANEL_UPDATE_BUTTON })
	TitanSpec_output(TITAN_SPEC_ID.." by Rothpada (Shadow Council)");
	TitanPanelPluginHandle_OnUpdate({TITAN_SPEC_ID, TITAN_PANEL_UPDATE_ALL})
end

function TitanPanelTitanSpecButton_OnClick(self, button)
	if ( button == "LeftButton" ) then
		if ( GetNumTalentGroups() > 1 ) then
			-- switch specs
			SetActiveTalentGroup(mod(GetActiveTalentGroup(), GetNumTalentGroups()) + 1);
		end
		
		TitanPanelPluginHandle_OnUpdate({TITAN_SPEC_ID, TITAN_PANEL_UPDATE_ALL})
	end
end

function TitanPanelTitanSpecButton_OnEvent(self, event, ...)
	TitanSpec_debug(event);
	TitanPanelPluginHandle_OnUpdate({TITAN_SPEC_ID, TITAN_PANEL_UPDATE_ALL})
end

function TitanPanelTitanSpecButton_GetButtonText(id)
	local label = L["TITAN_SPEC_SPEC"].." "..GetActiveTalentGroup()..": ";

    -- Build the button text
	local b1,_,t1,_ = GetTalentTabInfo(1, false, false);
	local b2,_,t2,_ = GetTalentTabInfo(2, false, false);
	local b3,_,t3,_ = GetTalentTabInfo(3, false, false);
	
	local specText = TitanGetVar(TITAN_SPEC_ID, "SpecText");
	local specTree = TitanGetVar(TITAN_SPEC_ID, "SpecTree");
		
	local buildName = L["TITAN_SPEC_HYBRID"];
	if ( specText[GetActiveTalentGroup()] ~= nil ) then
		buildName = specText[GetActiveTalentGroup()];
	else
		if ( t1 > 40 ) then
			buildName = b1;
		elseif ( t2 > 40 ) then
			buildName = b2;
		elseif ( t3 > 40 ) then
			buildName = b3;
		end	
	end
	if ( t1 > 40 ) then
		specTree[GetActiveTalentGroup()] = b1;
	elseif ( t2 > 40 ) then
		specTree[GetActiveTalentGroup()] = b2;
	elseif ( t3 > 40 ) then
		specTree[GetActiveTalentGroup()] = b3;
	end
	
	if ( specText[GetActiveTalentGroup()] == nil ) then
		-- Save it out for next time
		specText[GetActiveTalentGroup()] = buildName;
		TitanSetVar(TITAN_SPEC_ID, "SpecText", specText);
		TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
	end
		
	local buildText = TitanGetVar(TITAN_SPEC_ID, "BuildText");
	buildText[GetActiveTalentGroup()] = format("%d/%d/%d",t1,t2,t3);

	TitanSetVar(TITAN_SPEC_ID, "BuildText", buildText);
	TitanSetVar(TITAN_SPEC_ID, "SpecTree", specTree);

	if ( TitanGetVar(TITAN_SPEC_ID, "ShowNumbers") ) then
    		return label, format("%s (%s)", buildName, buildText[GetActiveTalentGroup()]);
	end
	return label, buildName;
end

function TitanPanelTitanSpecButton_GetTooltipText(self)
	local texts = TitanGetVar(TITAN_SPEC_ID, "SpecText");
	local builds = TitanGetVar(TITAN_SPEC_ID, "BuildText");
	local trees = TitanGetVar(TITAN_SPEC_ID, "SpecTree");
	
	local t1 = "?"; local t2 = "?";
	
	if ( texts[1] ~= nil ) then 
		t1 = L["TITAN_SPEC_PRIMARY_SPEC"]..format("%s - %s (%s)", texts[1], trees[1], builds[1]); 
	end
	if ( texts[2] ~= nil ) then 
		t2 = L["TITAN_SPEC_SECONDARY_SPEC"]..format("%s - %s (%s)", texts[2], trees[2], builds[2]); 
	end
	
	if ( GetActiveTalentGroup() == 1 ) then
		t1 = TitanSpec_getColouredText(t1, "green");
		t2 = TitanSpec_getColouredText(t2, "yellow");
	else
		t1 = TitanSpec_getColouredText(t1, "yellow");
		t2 = TitanSpec_getColouredText(t2, "green");
	end

	return t1.."\n"..t2.."\n";
end

function TitanPanelRightClickMenu_PrepareTitanSpecMenu()
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[TITAN_SPEC_ID].menuText);
	TitanPanelRightClickMenu_AddSpacer();
	
	local info = {};
	info.text = L["TITAN_SPEC_RENAME"];
	info.func = function()
		StaticPopupDialogs["TITAN_SPEC_RENAME"] = {
		  text = L["TITAN_SPEC_RENAME_TEXT"],
		  button1 = L["TITAN_SPEC_ACCEPT"],
		  button2 = L["TITAN_SPEC_CANCEL"],
		  OnAccept = function()
		  	local newName = getglobal(this:GetParent():GetName().."EditBox"):GetText();
		  	local txt = TitanGetVar(TITAN_SPEC_ID, "SpecText");
			txt[GetActiveTalentGroup()] = newName;
			TitanSetVar(TITAN_SPEC_ID, "SpecText", txt);
			TitanPanelButton_UpdateButton(TITAN_SPEC_ID);
		  end,
		  timeout = 0,
		  whileDead = 1,
		  hideOnEscape = 1,
		  hasEditBox = 1
		};
		StaticPopup_Show("TITAN_SPEC_RENAME");
	end
	UIDropDownMenu_AddButton(info);
	
	info = {};
	info.text = L["TITAN_SPEC_RESET"];	
	info.func = function()
		TitanSetVar(TITAN_SPEC_ID, "SpecText", {});
		TitanSetVar(TITAN_SPEC_ID, "SpecTree", {});
		TitanSetVar(TITAN_SPEC_ID, "BuildText", {});
	end
	UIDropDownMenu_AddButton(info);
	
	TitanPanelRightClickMenu_AddSpacer();
	
	TitanPanelRightClickMenu_AddToggleVar("Show build", TITAN_SPEC_ID, "ShowNumbers");
	TitanPanelRightClickMenu_AddToggleLabelText(TITAN_SPEC_ID);
end

-- ************************************************************************** --
-- ******************************** User Functions ************************** --
-- ************************************************************************** --

function TitanSpec_debug(msg)
	if TitanSpec_IS_DEBUGGING then
		TitanSpec_output(msg);
	end
end

function TitanSpec_output(msg)
    DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 1, 1);
end


function TitanSpec_getColouredText(msg, colour)
	local colourCode = "";
	
	if ( colour == nil or colour == "default" ) then
		return msg;
	elseif ( colour == "red" ) then
		colourCode = "|cffff0000";
	elseif ( colour == "green" ) then
		colourCode = "|cff00ff00";
	elseif ( colour == "blue" ) then
		colourCode = "|cff0000ff";
	elseif ( colour == "yellow" ) then
		colourCode = "|cffffff00";
	elseif ( colour == "white" ) then
		colourCode = "|cffffffff";
	elseif ( colour == "black" ) then
		colourCode = "|cff000000";
	else
		colourCode = "|cff"..colour;
	end
        
	return colourCode..msg..FONT_COLOR_CODE_CLOSE;
end
